static char *sccsid = "@(#) samples/oltp/amqsxae0.c, samples, p510b, p510-03-991110  1.10.2.1 99/05/31 00:32:54";
/*********************************************************************/
/*                                                                   */
/* Module Name: amqsxae0.c                                           */
/*                                                                   */
/* Description: MQSeries sample Encina program                       */
/*                                                                   */
/*  Statement:     Licensed Materials - Property of IBM              */
/*                                                                   */
/*                 04L1773, 5765-B73                                 */            
/*                 04L1802, 5639-B42                                 */            
/*                 04L1788, 5765-B74                                 */            
/*                 04L1816, 5765-B75                                 */            
/*                 04L1830, 5639-B43                                 */            
/*                 (C) Copyright IBM Corp. 1994, 1999                */
/*                                                                   */
/*********************************************************************/
/*              IMPORTANT INFORMATION WHEN USING THIS PROGRAM        */
/*                        ON WINDOWS* NT PLATFORM                    */
/*                                                                   */
/* If this program is to be used on a Windows* NT platform then you  */
/* must ensure that a symbolic constant named "WIN32" is defined at  */
/* compile time.                                                     */
/*                                                                   */
/* This can be done by specifying a compiler option of  -DWIN32.     */
/*                                                                   */
/* The symbolic constant "WIN32" is referenced twice in the code by  */
/* the statement "#if defined(WIN32)".                               */
/*                                                                   */
/* (* Windows is a trademark of Microsoft Corporation.)              */
/*                                                                   */
/*                        END OF INFORMATION                         */
/*********************************************************************/
/* Function:                                                         */
/*                                                                   */
/* amqsxae0.c is an MQSeries sample of an Encina transaction working */
/* with MQSeries.                                                    */
/*                                                                   */
/* This program puts 10 messages to the queue, backing out the odd   */
/* numbered messages and committing the even numbered messages.      */
/* The message is a 4-byte number.                                   */
/*                                                                   */
/* For information on compiling and linking this sample program,     */
/* refer to the "Building an MQSeries application" and               */
/* "Sample MQSeries programs" parts of the                           */
/* "MQSeries Application Programming Guide".                         */
/*                                                                   */
/* amqsxae0.c contains the following functions:                      */
/*  main                 Encina entry point                          */
/*                                                                   */
/*********************************************************************/

/*********************************************************************/
/* Includes                                                          */
/*********************************************************************/

/*********************************************************************/
/* Select compiler specific header files                             */
/*********************************************************************/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef min
  #define min(a,b)        (((a) < (b)) ? (a) : (b))
#endif

/*********************************************************************/
/* include Encina header files                                       */
/*********************************************************************/
#include <tc/tc.h>
#include <tmxa/tmxa_status.h>
#include <tmxa/tmxa.h>

/*********************************************************************/
/* Register module with Tran-C                                       */
/*********************************************************************/
inModule("MFXAtest");

/*********************************************************************/
/* includes for MQI                                                  */
/*********************************************************************/
#include <cmqc.h>
#include <cmqcfc.h>

/*********************************************************************/
/* prototype the dummy recovery services                             */
/*********************************************************************/
unsigned int myAppl_ApplCallBack(void * arg);
void RecPrepare(tran_serviceId_t serviceid,
                tran_tid_t tid,
                tran_logRecord_t prepareRecord,
                tran_recOptimization_t *optimise);
void RecCommit(tran_serviceId_t serviceid,
               tran_tid_t tid,
               tran_recOptimization_t *optimise);
void RecWrite(tran_serviceId_t serviceid,
              tran_tid_t tid,
              tran_logRecord_t record,
              tran_recOptimization_t *optimise);
void RecFinished(tran_serviceId_t serviceid,
                 tran_tid_t tid,
                 tran_recOptimization_t *optimise);
void RecAbort(tran_serviceId_t serviceid,
              tran_tid_t tid,
              tran_recOptimization_t *optimise);
void RecBlock(tran_serviceId_t serviceid,
              tran_tid_t tid,
              tran_blockId_t blockid,
              unsigned long  microsecondsUntilAlarm);
void RecWakeup(tran_serviceId_t serviceId,
               tran_tid_t tid,
               tran_blockId_t blockId);
void RecActive(tran_serviceId_t serviceid,
               tran_tid_t tid,
               int activeNow);

/*********************************************************************/
/* NOTE: IN THE FOLLOWING DECLARATIONS FOR "MQRMIXASwitch" and       */
/*       "MQRMIXASwitchDynamic", IF COMPILING FOR A WINDOWS* NT      */
/*       PLATFORM, YOU MUST ENSURE THAT THE SYMBOLIC CONSTANT "WIN32"*/
/*       IS DEFINED AT COMPILE TIME.                                 */
/*                                                                   */
/* Either MQRMIXASwitch or MQRMIXASwitchDynamic can be used.         */
/*                                                                   */
/* NOTE: WHICHEVER SWITCH YOU USE MUST BE NAMED ON THE SUBSEQUENT    */
/*       ASSIGNMENT TO THE  "switchP"  variable.                     */
/*       THE DEFAULT ASSIGNMENT VALUE IS "MQRMIXASwitch".            */
/*                                                                   */
/*********************************************************************/

#if defined(WIN32)
__declspec(dllimport) struct xa_switch_t MQRMIXASwitch;
__declspec(dllimport) struct xa_switch_t MQRMIXASwitchDynamic;
#else
extern struct xa_switch_t MQRMIXASwitch;
extern struct xa_switch_t MQRMIXASwitchDynamic;
#endif

/*********************************************************************/
/* Define the name of the queue manager (use default)                */
/*********************************************************************/
#define MYQMGR ""

/*********************************************************************/
/* the program itself                                                */
/*********************************************************************/

#if defined(WIN32)
int __cdecl main(int argc, char **argv)
#else
int         main(int argc, char **argv)
#endif
{
   /******************************************************************/
   /* Declare MQI structures needed                                  */
   /******************************************************************/
   MQOD     od = {MQOD_DEFAULT};      /* Object Descriptor           */
   MQPMO   pmo = {MQPMO_DEFAULT};     /* get message options         */

   /******************************************************************/
   /* note, uses defaults where it can                               */
   /******************************************************************/
   MQHCONN  Hcon;                     /* connection handle           */
   MQHOBJ   Hobj;                     /* object handle               */
   MQLONG   CompCode;                 /* completion code             */
   MQLONG   Reason;                   /* reason code                 */
   MQLONG   CReason;                  /* reason code for MQCONN      */
   char     QMName[50];               /* queue manager name          */
   MQMD     PMsgDescriptor= { MQMD_DEFAULT }; /* message descriptor  */
   int      flret;                    /* used in flush               */
   int      ch;                       /* used for pause              */
   int      i2;                       /* counter                     */
   char    *openInfo = MYQMGR;        /* used in xa_open calls       */
   char    *closeInfo = "Not used";   /* used in xa_close calls      */

   struct   xa_switch_t *switchP = &MQRMIXASwitch;
                                      /* MQ Series XA switch         */

   int      threadSupport = TMXA_SERIALIZE_ALL_XA_OPERATIONS;
                                      /* the kind of thread support  */
   int      rmid;                     /* resource manager id         */
   tmxa_status_t    tmxarc;           /* registration ret code       */
   tran_serviceId_t service_id;       /* service identifier          */
   struct tagmsg {
                    int number;
                 } msg;

   /******************************************************************/
   /* Register function with Tran-C                                  */
   /******************************************************************/
   inFunction("main");

   /******************************************************************/
   /* Inform the TM-XA Service of a resource manager instance        */
   /******************************************************************/
   printf("about to register RMI\n");
   tmxarc = tmxa_RegisterRMI(openInfo,
                             closeInfo,
                             switchP,
                             threadSupport,
                             &rmid);

   switch (tmxarc)
   {
     case TMXA_SUCCESS:
       printf("registered sucessfully\n");
       break;
     case TMXA_ALREADY_INITIALIZED:
       printf("already initialised\n");
       break;
     case TMXA_INVALID_PARAM:
       printf("invalid parm\n");
       goto exit;
       break;
     case TMXA_TOO_MANY_INSTANCES_REGISTERED:
       printf("too many instances registered\n");
       goto exit;
       break;
     case TMXA_NO_MORE_MEMORY:
       printf("no more memory\n");
       goto exit;
       break;
     default:
       printf("unknown tmxarc, %ld\n",tmxarc);
       goto exit;
       break;
   }

   /******************************************************************/
   /* register callback for the initialization phase                 */
   /******************************************************************/
   printf("about to register applcallback\n");
   registerApplCallback(APPL_INIT_CALLBACK,  /* init phase           */
                        myAppl_ApplCallBack, /* callback function    */
                        (void *) 0);         /* functions parameters */


   /******************************************************************/
   /* Initialise Tran-C run-time system                              */
   /******************************************************************/
   printf("about to preinitTC\n");
   if (preInitTC())
   {
     /****************************************************************/
     /* Initialise the recovery service interface                    */
     /****************************************************************/
     printf("about to recinit\n");
     tran_RecInit(RecWrite,            /* proc to write log records  */
                  RecPrepare,          /* proc to prepare transaction*/
                  RecCommit,           /* to deliver a commit outcome*/
                  RecAbort,            /* to deliver an abort outcome*/
                  RecFinished,         /* dispose of log records     */
                  RecActive,           /* to say whether trans active*/
                  &service_id);        /* service identifier         */

     /****************************************************************/
     /* provide block and wakeup functions                           */
     /****************************************************************/
     printf("about to recblockfunctions\n");
     tran_RecBlockFunctions(service_id,/* service identifier         */
                            RecBlock,  /* proc to block waiting      */
                            RecWakeup);/* to make RecBlock return    */
   }
   else
   {
     printf("Spooky but...we've already initialised\n");
   }

   /******************************************************************/
   /* Finish the Tran-C initialization sequence                      */
   /******************************************************************/
   printf("about to postinitTC\n");
   postInitTC();


   /******************************************************************/
   /* Connect to queue manager                                       */
   /******************************************************************/
   strcpy(QMName,MYQMGR);
   printf("press enter to connect\n");
   ch = getchar();
   printf("about to MQCONN\n");
   MQCONN(QMName,                      /* queue manager              */
          &Hcon,                       /* connection handle          */
          &CompCode,                   /* completion code            */
          &CReason);                   /* reason code                */

   /******************************************************************/
   /* report reason and stop if it failed                            */
   /******************************************************************/
   if (CompCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %ld\n", CReason);
     exit(CReason);
   }

   /******************************************************************/
   /* Initialize object descriptor for subject queue                 */
   /* Open the test queue. We have to do this first to get it's      */
   /* name, since we're using a model queue.                         */
   /******************************************************************/
   strcpy(od.ObjectName, "SYSTEM.DEFAULT.MODEL.QUEUE");

   /******************************************************************/
   /* Open the test queue for input; exclusive or shared.  Use of    */
   /* the queue is controlled by the queue definition here           */
   /******************************************************************/
   MQOPEN(Hcon,                        /* connection handle          */
          &od,                         /* object descriptor for queue*/
           MQOO_OUTPUT                 /* open queue for putting     */
         + MQOO_FAIL_IF_QUIESCING,     /* but not if MQM stopping    */
          &Hobj,                       /* object handle              */
          &CompCode,                   /* completion code            */
          &Reason);                    /* reason code                */

   /******************************************************************/
   /* report reason, if any; stop if failed                          */
   /******************************************************************/
   if (Reason != MQRC_NONE)
   {
     printf("MQOPEN (%s) ==> %ld\n", od.ObjectName, Reason);
   }

   /******************************************************************/
   /* set the put message options to be under syncpoint              */
   /******************************************************************/
   pmo.Options = pmo.Options + MQPMO_SYNCPOINT;

   /******************************************************************/
   /* Put the 10 messages, alternately commiting or rolling them     */
   /* back                                                           */
   /******************************************************************/
   printf("about to do the loop\n");
   for(i2 = 0;i2 < 10;i2++)
   {
     transaction                       /* start the transaction      */
     {
       /**************************************************************/
       /* print the transaction id                                   */
       /**************************************************************/
       printf("transaction %d\n", getTid());

       printf("about to put, press enter to continue\n");
       ch = getchar();

       /**************************************************************/
       /* put the message, putting the loop counter number as the    */
       /* data                                                       */
       /**************************************************************/
       msg.number = i2;
       MQPUT(Hcon,                     /* connection handle          */
             Hobj,                     /* object handle              */
             &PMsgDescriptor,          /* message descriptor         */
             &pmo,                     /* put message options        */
             (long int)sizeof(msg),    /* size of message data       */
             &msg,                     /* message data               */
             &CompCode,                /* completion code            */
             &Reason );                /* reason code                */

       /**************************************************************/
       /* if put failed then close, disconnect and exit              */
       /**************************************************************/
       if (CompCode)
       {
         printf("MQPUT Hex  Reason code    = %x\n", Reason);
         printf("           Completion code= %d\n", CompCode);
         flret = fflush(stdout);

         printf( "DO MQCLOSE -MQPUT Failed\n" );

         MQCLOSE(Hcon,                 /* connection handle          */
                 &Hobj,                /* object handle              */
                 0,                    /* options                    */
                 &CompCode,            /* completion code            */
                 &Reason);             /* reason code                */

         printf("MQCLOSE Hex  Reason code    = %x\n", Reason);
         printf("             Completion code= %d\n", CompCode);
         puts("***\n");
         flret = fflush(stdout);

         printf( "DO MQDISC\n" );

         MQDISC(&Hcon,
                &CompCode,
                &Reason );

         printf("MQDISC  Hex  Reason code    = %x\n", Reason);
         printf("             Completion code= %d\n", CompCode);
         puts("***\n");
         flret = fflush(stdout);
         puts("Bye...\n\n");
         goto commit;
       }

       printf("put done, press enter to continue\n");
       ch = getchar();

       /**************************************************************/
       /* If it is an odd numbered transaction then roll back the    */
       /* message, otherwise commit it                               */
       /**************************************************************/
       if (i2 % 2)
           abort("Odd Numbered Transactions are aborted...");
commit:
       ;
     }

     /****************************************************************/
     /* define what to do at commit and abort time                   */
     /****************************************************************/
     onCommit
       printf("\t(Transaction committed)\n");
     onAbort
       printf("Aborted in module : %s\n\t%s\n",
              abortModuleName(),
              abortReason());
   }

   /******************************************************************/
   /* Pause before closing so that you can look at the queue         */
   /* using some other application to verify that only then even     */
   /* numbered messages are on the queue.                            */
   /******************************************************************/
   printf("press enter to MQCLOSE\n");
   ch = getchar();
   MQCLOSE(Hcon,                       /* connection handle          */
           &Hobj,                      /* object handle              */
           0,                          /* options                    */
           &CompCode,                  /* completion code            */
           &Reason);                   /* reason code                */

   printf("MQCLOSE Hex  Reason code    = %x\n", Reason);
   printf("             Completion code= %d\n", CompCode);
   puts("***\n");
   flret = fflush(stdout);

   /******************************************************************/
   /* Disconnect from MQM  (unless previously connected)             */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
    printf("about to MQDISC\n");
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */
     /****************************************************************/
     /* report reason, if any                                        */
     /****************************************************************/
     if (Reason != MQRC_NONE)
     {
       printf("MQDISC ended with reason code %ld\n", Reason);
     }
   }
exit:
   ;

 /********************************************************************/
 /*                                                                  */
 /* End of main function                                             */
 /*                                                                  */
 /********************************************************************/
   exitTC(0);  /* terminates main(), subsequent return not executed  */
   return 0;   /*   but consistent with definition of main()         */
 }


/*********************************************************************/
/* the application callback function used in initialization          */
/*********************************************************************/
unsigned int myAppl_ApplCallBack(void * arg)
{
  tmxa_status_t tmxaStatus;
  /*******************************************************************/
  /* initialize the TM-XA Service                                    */
  /*******************************************************************/
  tmxaStatus = tmxa_Init();
  if (tmxaStatus != TMXA_SUCCESS)
  {
    fprintf(stderr, "Error return from tmxa_init: %d\n", tmxaStatus);
    return 0;
  }
  return 1;
}

/*********************************************************************/
/*  Dummy recovery service functions that just print out data        */
/* saying that the functions have been entered                       */
/*********************************************************************/
void RecPrepare(tran_serviceId_t serviceid,
                tran_tid_t tid,
                tran_logRecord_t prepareRecord,
                tran_recOptimization_t *optimise)
{
        printf("RecPrepare(sid=%d,tid=%d,optimise=0x%x)\n",
                     serviceid, tid, optimise);
        tran_RecAcknowledge(serviceid,tid);
}

void RecCommit(tran_serviceId_t serviceid,
               tran_tid_t tid,
               tran_recOptimization_t *optimise)
{
         printf("RecCommit(sid=%d,tid=%d,optim=0x%x)\n",
                                serviceid,tid,optimise);
}

void RecWrite(tran_serviceId_t serviceid,
              tran_tid_t tid,
              tran_logRecord_t record,
              tran_recOptimization_t *optimise)
{
         printf("RecWrite(sid=%d,tid=%d,optim=0x%x)\n",
                    serviceid,tid,optimise);
}

void RecFinished(tran_serviceId_t serviceid,
                 tran_tid_t tid,
                 tran_recOptimization_t *optimise)
{
        printf("RecFinished(sid=%d,tid=%d,optim=0x%x)\n",
                     serviceid,tid,optimise);
        tran_RecAcknowledge(serviceid,tid);
}

void RecAbort(tran_serviceId_t serviceid,
              tran_tid_t tid,
              tran_recOptimization_t *optimise)
{
        printf("RecAbort(sid=%d,tid=%d,optim=0x%x)\n",
                     serviceid,tid,optimise);
        tran_RecAcknowledge(serviceid,tid);
}

void RecBlock(tran_serviceId_t serviceid,
              tran_tid_t tid,
              tran_blockId_t blockid,
              unsigned long microsecondsUntilAlarm)
{
         printf("RecBlock(sid=%d,tid=%d,blockid=%d)\n",
                     serviceid,tid,blockid);
}

void RecWakeup(tran_serviceId_t serviceId,
               tran_tid_t tid,
               tran_blockId_t blockId)
{
         printf("RecWakeup\n");
}

void RecActive(tran_serviceId_t serviceid,
               tran_tid_t tid,
               int activeNow)
{
         printf("RecActive(sid=%d,tid=%d,active=%d)\n",
                     serviceid,tid,activeNow);
}

